'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Cache {

  constructor(logger) {

    this.logger = logger;

    let redis;

    if (process.env.NODE_ENV === 'teest') {
      redis = require('redis-mock');
    } else {
      redis = require('redis');
    }

    this.client = redis.createClient(process.env.REDIS_PORT, process.env.REDIS_HOST);
  }

  invalidate(key) {

    return new _promise2.default((resolve, reject) => {

      this.client.del(key, err => {

        if (err) {
          reject(err);
        }

        resolve();
      });
    });
  }

  invalidateApplication(applicationId) {

    return new _promise2.default((resolve, reject) => {

      this.client.keys(`${applicationId}:*`, async (err, keys) => {

        if (err) {
          reject(err);
        }

        for (const key of keys) {

          await this.invalidate(key);
        }

        resolve();
      });
    });
  }

  invalidateAll() {

    return new _promise2.default((resolve, reject) => {

      this.client.flushall(err => {

        if (err) {
          reject(err);
        }

        resolve();
      });
    });
  }

  get(key) {

    return new _promise2.default((resolve, reject) => {

      this.client.get(key, (err, value) => {

        if (err) {
          reject(err);
        }

        try {
          const json = JSON.parse(value);
          resolve(json);
        } catch (e) {
          resolve(value);
        }
      });
    });
  }

  expire(key, ttl) {

    return new _promise2.default((resolve, reject) => {

      this.client.expire(key, ttl, err => {

        if (err) {
          reject(err);
        }

        resolve();
      });
    });
  }

  set(key, value, ttl) {

    return new _promise2.default((resolve, reject) => {

      this.client.set(key, value, async err => {

        if (err) {
          reject(err);
        }

        if (ttl) {

          await this.expire(key, ttl);
        }

        resolve();
      });
    });
  }

  cacheRequest(applicationId, key, res, ttl) {

    const send = res.send.bind(res);

    // Wrap res.send so we can intercept and cache the response
    res.send = async body => {

      // Send response to client
      const response = send(body);

      try {

        // Save response to cache
        await this.set(`${applicationId}:${key}`, body, ttl);
      } catch (err) {

        this.logger.error('Error setting cache', { err, applicationId, key, res, ttl });
      }

      return response;
    };
  }

}

exports.default = Cache;
//# sourceMappingURL=cache.js.map