'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = cache;

var _cache = require('./cache');

var _cache2 = _interopRequireDefault(_cache);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * Cache middleware layer on rethinkdb.
 *
 * @public
 * @param {Object} ttl - how many seconds cache should stay valid for, default 10 minutes
 */
function cache(ttl = 600) {

  let cache;

  return async function (req, res, next) {

    if (process.env.SKIP_CACHE) {
      return next();
    }

    const logger = req.logger || console;

    if (!cache) {
      cache = new _cache2.default(logger);
    }

    const key = req.originalUrl;
    const applicationId = req.headers.applicationid;

    try {

      const value = await cache.get(`${applicationId}:${key}`);

      if (value === null) {

        // URL not found in cache, process request normally and cache response
        logger.debug(`Cache Miss: ${key}`);
        cache.cacheRequest(applicationId, key, res, ttl);
        next();
      } else {

        // URL found in cache, return cached response
        logger.debug(`Cache Hit: ${key}`);
        res.send(value);
      }
    } catch (e) {

      logger.error(`Cache Error: ${e}`);
      next();
    }
  };
}
//# sourceMappingURL=middleware.js.map